@@ -1,9 +1,10 @@
 import asyncio
 import re
+import sys
 from collections import defaultdict
+from contextlib import suppress
 from typing import DefaultDict, List, Optional, Tuple, Union
 
-import aiohttp
 import arrow
 from aiohttp import ClientSession
 from diskcache import Cache
@@ -11,6 +12,7 @@ from nonebot.adapters.onebot.v11 import (
     ActionFailed,
     Bot,
     GroupMessageEvent,
+    LifecycleMetaEvent,
     Message,
     MessageEvent,
     PrivateMessageEvent,
@@ -18,7 +20,7 @@ from nonebot.adapters.onebot.v11 import (
 from nonebot.log import logger
 from nonebot.matcher import Matcher
 from nonebot.params import CommandArg
-from nonebot.plugin.on import on_command, on_message
+from nonebot.plugin.on import on_command, on_message, on_metaevent
 from nonebot.rule import Rule
 from PicImageSearch import Network
 from tenacity import AsyncRetrying, stop_after_attempt, stop_after_delay
@@ -29,12 +31,32 @@ from .config import config
 from .ehentai import ehentai_search
 from .iqdb import iqdb_search
 from .saucenao import saucenao_search
-from .utils import handle_img, handle_reply_msg
+from .utils import get_bot_friend_list, handle_img, handle_reply_msg
 
 sending_lock: DefaultDict[Tuple[Union[int, str], str], asyncio.Lock] = defaultdict(
     asyncio.Lock
 )
 
+# issue 30
+if sys.version_info >= (3, 8) and sys.platform == "win32":
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+
+
+async def check_first_connect(_: LifecycleMetaEvent) -> bool:
+    return True
+
+
+start_metaevent = on_metaevent(rule=check_first_connect, temp=True)
+
+
+@start_metaevent.handle()
+async def _(bot: Bot) -> None:
+    if not config.saucenao_api_key:
+        await bot.send_private_msg(
+            user_id=int(list(config.superusers)[0]),
+            message="请配置 saucenao_api_key ，否则无法正常使用搜图功能",
+        )
+
 
 def has_images(event: MessageEvent) -> bool:
     message = event.reply.message if event.reply else event.message
@@ -53,7 +75,7 @@ async def to_me_with_images(bot: Bot, event: MessageEvent) -> bool:
         return (
             has_image
             and (event.to_me or at_me)
-            and "再搜" in event.message.extract_plain_text()
+            and "搜图" in event.message.extract_plain_text()
         )
     return has_image and (event.to_me or at_me)
 
@@ -63,10 +85,12 @@ IMAGE_SEARCH_MODE = on_command("搜图", priority=5)
 
 
 @IMAGE_SEARCH_MODE.handle()
-async def handle_first_receive(matcher: Matcher, args: Message = CommandArg()) -> None:
+async def handle_first_receive(
+    event: MessageEvent, matcher: Matcher, args: Message = CommandArg()
+) -> None:
     mode, purge = get_args(args)
     matcher.state["ARGS"] = (mode, purge)
-    if [i for i in args if i.type == "image"]:
+    if has_images(event):
         matcher.set_arg("IMAGES", args)
 
 
@@ -90,16 +114,17 @@ async def image_search(
             with attempt:
                 if mode == "a2d":
                     result = await ascii2d_search(url, client, hide_img)
-                elif mode == "iqdb":
-                    result = await iqdb_search(url, client, hide_img)
                 elif mode == "ex":
                     result = await ehentai_search(url, client, hide_img)
+                elif mode == "iqdb":
+                    result = await iqdb_search(url, client, hide_img)
                 else:
                     result = await saucenao_search(url, mode, client, hide_img)
-                upsert_cache(_cache, image_md5, mode, result)
+                    # 仅对涉及到 saucenao 的搜图结果做缓存
+                    upsert_cache(_cache, image_md5, mode, result)
     except Exception as e:
-        logger.exception(f"❌️ 该图 [{url}] 搜图失败")
-        result = [f"❌️ 该图搜图失败\nE: {repr(e)}"]
+        logger.exception(f"该图 [{url}] 搜图失败")
+        result = [f"该图搜图失败\nE: {repr(e)}"]
     return result
 
 
@@ -109,7 +134,7 @@ async def get_universal_img_url(url: str) -> str:
     )
     final_url = re.sub(r"/\d+/+\d+-\d+-", "/0/0-0-", final_url)
     final_url = re.sub(r"\?.*$", "", final_url)
-    async with aiohttp.ClientSession() as session:
+    async with ClientSession() as session:
         async with session.get(final_url) as resp:
             if resp.status == 200:
                 return final_url
@@ -167,18 +192,26 @@ async def send_msg(
     if index is not None:
         message = f"第 {index + 1} 张图片的搜索结果：\n{message}"
     message = f"{handle_reply_msg(event.message_id)}{message}"
-    await bot.send_msg(
-        user_id=event.user_id if isinstance(event, PrivateMessageEvent) else 0,
-        group_id=event.group_id if isinstance(event, GroupMessageEvent) else 0,
-        message=message,
-    )
+    try:
+        await bot.send_msg(
+            user_id=event.user_id if isinstance(event, PrivateMessageEvent) else 0,
+            group_id=event.group_id if isinstance(event, GroupMessageEvent) else 0,
+            message=message,
+        )
+    except ActionFailed:
+        # 如果群消息发送失败，则尝试发送私聊消息（仅限好友）
+        if isinstance(event, GroupMessageEvent):
+            friend_list = await get_bot_friend_list(bot)
+            if event.user_id in friend_list:
+                with suppress(ActionFailed):
+                    await bot.send_msg(user_id=event.user_id, message=message)
 
 
 async def send_forward_msg(
     bot: Bot, event: MessageEvent, msg_list: List[str], index: Optional[int] = None
 ) -> None:
     if index is not None:
-        msg_list = [f"第 {index + 1} 张图片的搜索结果：\n"] + msg_list
+        msg_list = [f"第 {index + 1} 张图片的搜索结果："] + msg_list
     await bot.send_forward_msg(
         user_id=event.user_id if isinstance(event, PrivateMessageEvent) else 0,
         group_id=event.group_id if isinstance(event, GroupMessageEvent) else 0,
